/* Copyring (C) 2003 Jani Jaakkola, jjaakkol@cs.helsinki.fi.
 * This software is distributed under the GNU Libgrary General Public
 * License.
 */
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <nss.h>
#include <netdb.h>
#include <string.h>

/* Prototypes for the resolving functions in nss_dns.so */

enum nss_status
_nss_dns_gethostbyname2_r (const char *name, int af, 
			   struct hostent *result,
			   char *buffer, size_t buflen, 
			   int *errnop,int *h_errnop);
enum nss_status
_nss_dns_gethostbyname_r (const char *name, struct hostent *result,
                          char *buffer, size_t buflen, int *errnop,
                          int *h_errnop);

static enum nss_status 
check_result(int ret,
	     struct hostent *result, 
	     int *h_errnop) {
  /* JJ: here is the verisign hack. If the query
   * was a success, but was returned to the bad IP-address, we return 
   * NSS_STATUS_NOTFOUND anyway. And try hit _only_ the verisign hack
   * and nothing else.
   */
  /* TODO: Maybe this needs AF_INET6 support? */
  if ( ret==NSS_STATUS_SUCCESS &&
       result->h_addrtype==AF_INET &&
       result->h_length==4 &&
       result->h_addr_list[0]) {
    struct in_addr tmp;
    inet_aton("64.94.110.11",&tmp);
    if (memcmp(result->h_addr_list[0],&tmp,sizeof(tmp))==0) {
      // Resolved to verisign
      *h_errnop=HOST_NOT_FOUND;
      return NSS_STATUS_NOTFOUND;
    }
  }
  return ret;
}

enum nss_status
_nss_brokendns_gethostbyname2_r (const char *name, int af, 
			   struct hostent *result,
			   char *buffer, size_t buflen, 
				 int *errnop,int *h_errnop) {
  enum nss_status ret;
  ret=_nss_dns_gethostbyname2_r(name,af,result,buffer,buflen,
			    errnop,h_errnop);
  return check_result(ret,result,h_errnop);
}

enum nss_status
_nss_brokendns_gethostbyname_r (const char *name, struct hostent *result,
                          char *buffer, size_t buflen, int *errnop,
				int *h_errnop) {
  enum nss_status ret;
  ret=_nss_dns_gethostbyname_r(name,result,buffer,buflen,
			      errnop,h_errnop);
  return check_result(ret,result,h_errnop);
}


     
